#
# Copyright (C) 1997 Ken MacLeod
# See the file COPYING for distribution terms.
#
# $Id: Element.pm,v 1.2 1997/10/06 03:52:00 ken Exp $
#

# Internally, an SGML::Element is an array containing
#
#    [0] -- contents
#    [1] -- gi (name)
#    [2] -- attributes

package SGML::Element;

use strict;

=head1 NAME

SGML::Element - an element of an SGML, XML, or HTML document

=head1 SYNOPSIS

  $element->gi;
  $element->name;
  $element->attributes;
  $element->contents;

I<more to come, including iterators, C<new>, and C<as_string>>

=head1 DESCRIPTION

C<SGML::Element> objects are loaded by C<SGML::SPGrove>.  An
C<SGML::Element> contains a generic identifier, or name, for the
element, the elements attributes and the ordered contents of the
element.

C<$element-E<gt>gi> and C<$element-E<gt>name> are synonyms, they
return the generic identifier of the element.

C<$element-E<gt>attributes> returns a reference to a hash containing
the attributes of the element.  The keys of the hash are the attribute
names and the values are references to an array containing scalars or
C<SGML::SData> objects.

C<$element-E<gt>contents> returns a reference to an array containing
the children of the element.  The contents of the element may contain
other elements, scalars, or C<SGML::SData> or C<SGML:PI> objects.

=head1 AUTHOR

Ken MacLeod, ken@bitsko.slc.ut.us

=head1 SEE ALSO

perl(1), SGML::SPGrove(3), Text::EntityMap(3), SGML::SData(3),
SGML::PI(3).

=cut

sub contents {
    return $_[0]->[0];
}

sub gi {
    return $_[0]->[1];
}

sub name {
    return $_[0]->[1];
}

sub attributes {
    return $_[0]->[2];
}

sub accept {
    my ($self) = shift;
    my ($visitor) = shift;

    $visitor->visit_element ($self, @_);
}

sub children_accept {
    my ($self) = shift;
    my ($visitor) = shift;

    my ($ii);
    for ($ii = 0; $ii <= $#{$self->[0]}; $ii ++) {
	my ($child) = $self->[0][$ii];
	if (!ref ($child)) {
	    $visitor->visit_scalar ($child, @_);
	} else {
	    $self->[0][$ii]->accept ($visitor, @_);
	}
    }
}

1;
